// Wir schauen immer einen Teil der komplexen Zahlenebene an, bei
// der Realteil der komplexen Zahl auf die x-Achse gelegt wird, und
// der Imaginäreteil auf die y-Achse.
// z.B. xmin = -1.5; 
//      ymin = -.1; 
// Dann müssen wir noch die Größe des Ausschnitts festlegen, das geschieht
// über wh. Dieser Wert wird beim Vergrößern oder Verkleinern entsprechend
// verändert. (in MousePressed())
double xmin = -2.5; 
double ymin = -2; 
double wh = 4;

// maximale Anzahl Iterationen, die für die komplexe Zahl ausgeführt wird
// danach wird die Farbe anhand des Betrags der Zahl gewählt. Die Anzahl 
// kann verändert werden, entsprechend mehr Details sieht man dann in 
// manchen Teilen der Menge
int maxiterations = 30;
double massFuerUnendlich = 16;

//---------------------------------------------------------------------

int mandelbrotIteration(double x, double y) {

  // nun wird für eine komplexe Zahl bzw. einen Punkt der Ebene getestet, 
  // wohin er bei der Iteration geht. Je nachdem wird er dann später ein-
  // gefärbt
  double a = x;
  double b = y;
  int n = 0;
  while (n < maxiterations) {
    double aa = a * a;
    double bb = b * b;
    double twoab = 2.0 * a * b;
    a = aa - bb;
    b = twoab;
    if (aa + bb > massFuerUnendlich) {
       break;  // Bail
    }
    n++;
  }
  return n;
}

//------------------------------------------------

color colorize(int n) {
  // jedes Pixel wird entsprechend der Anzahl Iterationen eingefärbt
  // bis entweder der Wert der komplexen Zahl Null oder über einer
  // Schwelle ist
  if (n == maxiterations) return color(0, 0, 0);
  color col =  color(n*16 % 255);
  return col; 
}

//------------------------------------------------

void setup() {
  size(600, 600);
  noLoop();
}


void draw() {

  loadPixels();
  // x wird von xmin nach ixmax iteriert
  double xmax = xmin + wh;
  double ymax = ymin + wh;

  // die inkrmentelle Erhoehung für x und y
  double dx = (xmax - xmin) / (width);
  double dy = (ymax - ymin) / (height);

  double y = ymin;
  for (int j = 0; j < height; j++) {
    double x = xmin;
    for (int i = 0; i < width; i++) {
      int n = mandelbrotIteration(x, y);
      pixels[i+j*width] = colorize(n);
      if (((x<=0) && (x+dx>0)) || ((y<=0) && (y+dy>0)))
        pixels[i+j*width] = color(0,0,maxiterations);
      x += dx;
    }
    y += dy;
  }
  updatePixels();
}

//------------------------------------------------

void mousePressed() {

  // berechne, wo der Mauszeiger im gezeigten Ausschnitt der komplexen
  // Zahlenebene ist (Transformation Pixelposition nach (x,y))
  double x = xmin + wh*mouseX/width;
  double y = ymin + wh*mouseY/height;

  // linke Maustaste: verkleinere den gezeigten Ausschnitt um 0.66
  // rechte Taste: vergrößere den gezeigten Ausschnitt entsprechend
  if (mouseButton == LEFT) wh*=0.66; 
  else wh/=0.66;

  // bestimme den neuen gezeigten Ausschnitt
  xmin = x-wh/2.0;
  ymin = y-wh/2.0;

  redraw();
}  

//-------------------------------------------------

void keyPressed() {
  if (key == '+') maxiterations *= 1.2;
  if (key == '-') maxiterations /= 1.2;
  if (key == 's') save("mandelbrot.png");
  redraw();
}

//-------------------------------------------------
